package higa.sql.dataset;

import java.io.Serializable;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.CallableStatement;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;

import higa.util.EventMulticaster;

public class HDatabase implements Serializable {

  private String driver;
  private String connectionURL;
  private String userName;
  private String password;
  private boolean autoCommit = false;

  private EventMulticaster transactionCaster =
    new EventMulticaster();

  private transient Connection connection;

  public HDatabase() {}

  public void setDriver(String newDriver) {
    driver = newDriver;
    safeClose();
  }

  public String getDriver() {
    return driver;
  }

  public void setConnectionURL(String newConnectionURL) {
    connectionURL = newConnectionURL;
    safeClose();
  }

  public String getConnectionURL() {
    return connectionURL;
  }

  public void setUserName(String newUserName) {
    userName = newUserName;
    safeClose();
  }

  public String getUserName() {
    return userName;
  }

  public void setPassword(String newPassword) {
    password = newPassword;
    safeClose();
  }

  public String getPassword() {
    return password;
  }

  public void setAutoCommit(boolean newAutoCommit)
    throws SQLException {

    autoCommit = newAutoCommit;
    if (_open()) {
      connection.setAutoCommit(autoCommit);
    }
  }

  public boolean isAutoCommit() {
    return autoCommit;
  }

  public synchronized void open()
    throws Exception {

    Class.forName(driver).newInstance();

    connection = DriverManager.getConnection(
      connectionURL, userName, password);

    connection.setAutoCommit(autoCommit);
  }

  public synchronized void close() throws SQLException {
    if (isOpen()) {
      connection.close();
    }

    connection = null;
  }

  public synchronized void safeClose() {
    try {
      close();
    } catch (SQLException ex) {
      ex.printStackTrace();
    }
  }

  public void commit() throws SQLException {
    if (_open()) {
      connection.commit();
      fireCommit();
    }
  }

  protected void fireCommit() {
    TransactionEvent e =
      new TransactionEvent(this,
      TransactionEvent.COMMIT);

    try {
      transactionCaster.exceptionDispatch(e);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public void rollback() throws SQLException {
    if (_open()) {
      connection.rollback();
      fireRollback();
    }
  }

  protected void fireRollback() {
    TransactionEvent e =
      new TransactionEvent(this,
      TransactionEvent.ROLLBACK);

    try {
      transactionCaster.exceptionDispatch(e);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public Statement createStatement()
    throws SQLException {

    if (_open()) {
      return connection.createStatement();
    } else {
      return null;
    }
  }

  public PreparedStatement prepareStatement(
    String query) throws SQLException {

    if (_open()) {
      return connection.prepareStatement(query);
    } else {
      return null;
    }
  }

  public CallableStatement prepareCall(
    String query) throws SQLException {

    if (_open()) {
      return connection.prepareCall(query);
    } else {
      return null;
    }
  }

  public DatabaseMetaData getMetaData()
    throws SQLException {

    if (_open()) {
      return connection.getMetaData();
    } else {
      return null;
    }
  }

  public Connection getConnection() {
    if (_open()) {
      return connection;
    } else {
      return null;
    }
  }

  public void addTransactionListener(
    TransactionListener listener) {

    transactionCaster.add(listener);
  }

  public void removeTransactionListener(
    TransactionListener listener) {

    transactionCaster.remove(listener);
  }

  public boolean isOpen() {
    return connection != null;
  }

  private boolean _open() {
    if (isOpenNeed()) {
      try {
        open();
      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }

    return isOpen();
  }

  private boolean isOpenNeed() {
    if (connection == null &&
      driver != null &&
      connectionURL != null &&
      userName != null &&
      password != null) {

      return true;
    } else {
      return false;
    }
  }
}
